//----------------------------------------------------------------------------------------------------------------------
// Data Definitions
//----------------------------------------------------------------------------------------------------------------------
#define VK_0 0x30 // 0 key
#define VK_1 0x31 // 1 key
#define VK_2 0x32 // 2 key
#define VK_3 0x33 // 3 key
#define VK_4 0x34 // 4 key
#define VK_5 0x35 // 5 key
#define VK_6 0x36 // 6 key
#define VK_7 0x37 // 7 key
#define VK_8 0x38 // 8 key
#define VK_9 0x39 // 9 key
#define VK_A 0x41 // A key
#define VK_B 0x42 // B key
#define VK_C 0x43 // C key
#define VK_D 0x44 // D key
#define VK_E 0x45 // E key
#define VK_F 0x46 // F key
#define VK_G 0x47 // G key
#define VK_H 0x48 // H key
#define VK_I 0x49 // I key
#define VK_J 0x4A // J key
#define VK_K 0x4B // K key
#define VK_L 0x4C // L key
#define VK_M 0x4D // M key
#define VK_N 0x4E // N key
#define VK_O 0x4F // O key
#define VK_P 0x50 // P key
#define VK_Q 0x51 // Q key
#define VK_R 0x52 // R key
#define VK_S 0x53 // S key
#define VK_T 0x54 // T key
#define VK_U 0x55 // U key
#define VK_V 0x56 // V key
#define VK_W 0x57 // W key
#define VK_X 0x58 // X key
#define VK_Y 0x59 // Y key
#define VK_Z 0x5A // Z key
#define UI_DEFAULT  0x80 // Bit 7
#define UI_GROUPED  0x40 // Bit 6
#define UI_CALLBACK 0x20 // Bit 5
#define UI_COLORED  0x10 // Bit 4
#define UI_CHECKED  0x08 // Bit 3
#define UI_BITMAP   0x04 // Bit 2
#define UI_SELECT   0x02 // Bit 1
#define UI_SUBLEVEL 0x01 // Bit 0

struct AXLOCK
{
   UINT* p; // Pointer to the 32-bit color (ARGB) pixels
   WORD w;  // Width of the uncompressed texture (pitch)
   AXLOCK() {}
   AXLOCK(UINT* pC, WORD wP) : p(pC), w(wP) {}
   bool operator==(const AXLOCK& aLock) const
   {
      return p == aLock.p && w == aLock.w;
   }
   bool operator!=(const AXLOCK& aLock) const
   {
      return !(*this == aLock);
   }
   AXLOCK& operator=(const AXLOCK& aLock)
   {
      if(this != &aLock)
      {
         p = aLock.p;
         w = aLock.w;
      }
      return *this;
   }
};

struct AXMEMO
{
   WORD u;  // Window message type
   WORD v;  // Virtual-key code or LOWORD(wParam)
   short w; // Distance the wheel is rotated or HIWORD(wParam)
   short x; // Mouse cursor x-coordinate or key LOWORD(lParam)
   short y; // Mouse cursor y-coordinate or key HIWORD(lParam)
   AXMEMO() {}
   AXMEMO(WORD wU, WORD wV, short sW, short sX, short sY) : u(wU), v(wV), w(sW), x(sX), y(sY) {}
   bool operator==(const AXMEMO& aMemo) const
   {
      return u == aMemo.u && v == aMemo.v && w == aMemo.w && x == aMemo.x && y == aMemo.y;
   }
   bool operator!=(const AXMEMO& aMemo) const
   {
      return !(*this == aMemo);
   }
   AXMEMO& operator=(const AXMEMO& aMemo)
   {
      if(this != &aMemo)
      {
         u = aMemo.u;
         v = aMemo.v;
         w = aMemo.w;
         x = aMemo.x;
         y = aMemo.y;
      }
      return *this;
   }
};

struct AXMITE
{
   BYTE m; // Menu item status
   BYTE i; // Menu item index
   BYTE t; // Toolbar (toolbox or tree view) status
   BYTE e; // Toolbar or toolbox button item entry
   AXMITE() {}
   AXMITE(BYTE oM, BYTE oI, BYTE oT, BYTE oE) : m(oM), i(oI), t(oT), e(oE) {}
   bool operator==(const AXMITE& aMite) const
   {
      return m == aMite.m && i == aMite.i && t == aMite.t && e == aMite.e;
   }
   bool operator!=(const AXMITE& aMite) const
   {
      return !(*this == aMite);
   }
   AXMITE& operator=(const AXMITE& aMite)
   {
      if(this != &aMite)
      {
         m = aMite.m;
         i = aMite.i;
         t = aMite.t;
         e = aMite.e;
      }
      return *this;
   }
};

struct AXPLOT
{
   XMFLOAT3 p; // Vertex position in world space
   union
   {
      float v; // Vertex value for contour plot
      UINT c;  // Vertex 32-bit ARGB color data
   };
   XMFLOAT3 n; // Vertex normal in world space
   AXPLOT() {}
   AXPLOT(XMFLOAT3 vP, float fH, XMFLOAT3 vN) : p(vP), v(fH), n(vN) {}
   AXPLOT(XMFLOAT3 vP, UINT uI, XMFLOAT3 vN) : p(vP), c(uI), n(vN) {}
};

struct AXRECT
{
   short x; // X coordinate of the top-left corner
   short y; // Y coordinate of the top-left corner
   WORD w;  // Width of the rectangle
   WORD h;  // Height of the rectangle
   AXRECT() {}
   AXRECT(short sX, short sY, WORD wW, WORD wH) : x(sX), y(sY), w(wW), h(wH) {}
   bool operator==(const AXRECT& aRect) const
   {
      return x == aRect.x && y == aRect.y && w == aRect.w && h == aRect.h;
   }
   bool operator!=(const AXRECT& aRect) const
   {
      return !(*this == aRect);
   }
   AXRECT& operator=(const AXRECT& aRect)
   {
      if(this != &aRect)
      {
         x = aRect.x;
         y = aRect.y;
         w = aRect.w;
         h = aRect.h;
      }
      return *this;
   }
   AXRECT& operator=(const RECT& aRect)
   {
      x = short(aRect.left);
      y = short(aRect.top);
      w = WORD(aRect.right-aRect.left);
      h = WORD(aRect.bottom-aRect.top);
      return *this;
   }
   RECT* SetRect(RECT& aRect)
   {
      aRect.left = x;
      aRect.top = y;
      aRect.right = x+(long)w;
      aRect.bottom = y+(long)h;
      return &aRect;
   }
};

struct AXSIZE
{
   WORD w; // Width of the rectangle
   WORD h; // Height of the rectangle
   AXSIZE() {}
   AXSIZE(WORD wW, WORD wH) : w(wW), h(wH) {}
   bool operator==(const AXSIZE& aSize) const
   {
      return w == aSize.w && h == aSize.h;
   }
   bool operator!=(const AXSIZE& aSize) const
   {
      return !(*this == aSize);
   }
   AXSIZE& operator=(const AXSIZE& aSize)
   {
      if(this != &aSize)
      {
         w = aSize.w;
         h = aSize.h;
      }
      return *this;
   }
};
