#include "axbreeze.h"
void DrawWorldTriad(float fSize);

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   SetDisplayMode(1024, 768);

   WCHAR* pDefault = L"Default view (hCamera[\f#00ff00%d\f~#])";
   WCHAR* pView = pDefault;
   XMFLOAT3 fPosition[3], fTarget[3], fUpvector[3];
   UINT uCamera = 0, uColor[3] = { 0xffff0000, 0xff00ff00, 0xff0000ff };
   HANDLE hCamera[3];
   hCamera[0] = GetCameraCurrent();
   hCamera[1] = CreateCamera(2, 0, 2);
   hCamera[2] = CloneCamera(hCamera[0]);
   SetCameraFrustum(hCamera[1], 0.625, 0, 0.75);
   SetCameraPosition(hCamera[1], XMFLOAT3(-0.375, 0, 0));
   SetCameraViewWidth(hCamera[2], 1.25);
   SetCameraNearPlane(hCamera[2], 0.03125);
   SetCameraFarPlane(hCamera[2], 0.125);
   SetCameraRadialDistance(hCamera[2], GetCameraRadialDistance(hCamera[2])/4);
   for(UINT i = 0; i < 3; i++)
   {
      GetCameraPosition(hCamera[i], fPosition[i]);
      GetCameraTarget(hCamera[i], fTarget[i]);
      GetCameraUpvector(hCamera[i], fUpvector[i]);
   }

   // Enter the main loop
   while(Looping())
   {
      ClearDisplay();
      DrawText(10, 10, L"Press 0-9 to set camera view (Arrow/Control/Shift/W/A/S/D key to rotate)");
      DrawText(10, 40, L"Space bar to operate camera 0-2 (Enter key to set current camera 0-1)");
      BYTE oKey = GetVirtualKey();
      switch(oKey)
      {
         case VK_0:
         {
            for(UINT i = 0; i < 3; i++) SetCameraPosition(hCamera[i], fPosition[i], fTarget[i], fUpvector[i]);
            pView = pDefault;
            break;
         }
         case VK_1:
         {
            SetCameraTopView(hCamera[uCamera]);
            pView = L"SetCameraTopView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_2:
         {
            SetCameraBottomView(hCamera[uCamera]);
            pView = L"SetCameraBottomView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_3:
         {
            SetCameraFrontView(hCamera[uCamera]);
            pView = L"SetCameraFrontView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_4:
         {
            SetCameraBackView(hCamera[uCamera]);
            pView = L"SetCameraBackView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_5:
         {
            SetCameraLeftView(hCamera[uCamera]);
            pView = L"SetCameraLeftView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_6:
         {
            SetCameraRightView(hCamera[uCamera]);
            pView = L"SetCameraRightView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_7:
         {
            SetCameraIsometricView(hCamera[uCamera]);
            pView = L"SetCameraIsometricView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_8:
         {
            SetCameraReverseView(hCamera[uCamera]);
            pView = L"SetCameraReverseView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_9:
         {
            SetCameraFlipView(hCamera[uCamera]);
            pView = L"SetCameraFlipView\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
         case VK_UP:
         {
            PitchCamera(hCamera[uCamera], -0.03125);
            pView = L"PitchCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], -0.03125)";
            break;
         }
         case VK_DOWN:
         {
            PitchCamera(hCamera[uCamera], 0.03125);
            pView = L"PitchCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], 0.03125)";
            break;
         }
         case VK_LCONTROL:
         {
            YawCamera(hCamera[uCamera], -0.03125);
            pView = L"YawCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], -0.03125)";
            break;
         }
         case VK_RCONTROL:
         {
            YawCamera(hCamera[uCamera], 0.03125);
            pView = L"YawCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], 0.03125)";
            break;
         }
         case VK_LEFT:
         {
            RollCamera(hCamera[uCamera], -0.03125);
            pView = L"RollCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], -0.03125)";
            break;
         }
         case VK_RIGHT:
         {
            RollCamera(hCamera[uCamera], 0.03125);
            pView = L"RollCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], 0.03125)";
            break;
         }
         case VK_W:
         {
            RotateCamera(hCamera[uCamera], XMFLOAT3(-0.03125, 0, 0));
            pView = L"RotateCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], XMFLOAT3(-0.03125, 0, 0))";
            break;
         }
         case VK_S:
         {
            RotateCamera(hCamera[uCamera], XMFLOAT3(0.03125, 0, 0));
            pView = L"RotateCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], XMFLOAT3(0.03125, 0, 0))";
            break;
         }
         case VK_LSHIFT:
         {
            RotateCamera(hCamera[uCamera], XMFLOAT3(0, -0.03125, 0));
            pView = L"RotateCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, -0.03125, 0))";
            break;
         }
         case VK_RSHIFT:
         {
            RotateCamera(hCamera[uCamera], XMFLOAT3(0, 0.03125, 0));
            pView = L"RotateCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0.03125, 0))";
            break;
         }
         case VK_A:
         {
            RotateCamera(hCamera[uCamera], XMFLOAT3(0, 0, -0.03125));
            pView = L"RotateCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, -0.03125))";
            break;
         }
         case VK_D:
         {
            RotateCamera(hCamera[uCamera], XMFLOAT3(0, 0, 0.03125));
            pView = L"RotateCamera\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0.03125))";
            break;
         }
         case VK_SPACE:
         {
            uCamera = ++uCamera%3;
            pView = L"Controlled Camera = hCamera[\f#00ff00%d\f~#]";
            break;
         }
         case VK_RETURN:
         {
            if(uCamera < 2) SetCameraCurrent(hCamera[uCamera]);
            pView = L"SetCameraCurrent\f#ff00ff(hCamera[\f#00ff00%d\f#ff00ff])";
            break;
         }
      }
      for(UINT i = 0; i < 3; i++)
      {
         if(hCamera[i] == GetCameraCurrent())
         {
            PrintText(10, 70, L"GetCameraCurrent = hCamera[\f#00ff00%d\f~#]", i);
            continue;
         }
         DrawCameraFrustum(hCamera[i], uColor[i], 0.03125);
      }
      DrawWorldTriad(0.15625);
      XMFLOAT3 fH, fD, fU;
      GetCameraHorizon(hCamera[uCamera], fH);
      GetCameraDirection(hCamera[uCamera], fD);
      GetCameraUpvector(hCamera[uCamera], fU);
      PrintText(10, 620, L"GetCameraHorizon = \f#ff00ff%.1f, %.1f, %.1f", fH.x, fH.y, fH.z);
      PrintText(10, 650, L"GetCameraDirection = \f#ff00ff%.1f, %.1f, %.1f", fD.x, fD.y, fD.z);
      PrintText(10, 680, L"GetCameraUpvector = \f#ff00ff%.1f, %.1f, %.1f", fU.x, fU.y, fU.z);
      PrintText(512, 740, 0, 0, pView, uCamera);
      UpdateDisplay(30);
      if(oKey == VK_SPACE || oKey == VK_RETURN || oKey == VK_8 || oKey == VK_9) while(PollVirtualKey(oKey));
   }
}
