#include "axbreeze.h"
void DrawWorldTriad(float fSize);

void UpdateCamera()
{
   PrintText(10, 10, L"\f#ffff00FPS: \f#ff00ff%d", GetDisplayFrameRate());

   short sDx, sDy, sDz;
   const float fDs = 0.002f, fDt = 0.1f;
   static float fTheta = 0.5f*XM_PI, fPhi = 0.0f;
   float fR = 0.0f;

   GetMouseMove(MK_LBUTTON | MK_MBUTTON | MK_RBUTTON, sDx, sDy, sDz);
   fTheta -= fDs*sDy;
   fPhi -= fDs*sDx;
   fR = 5.0f*fDs*sDz;

   if(PollUpKey()) fTheta += fDt;
   if(PollDownKey()) fTheta -= fDt;
   if(PollLeftKey()) fPhi += fDt;
   if(PollRightKey()) fPhi -= fDt;
   if(PollShiftKey()) fR += fDt;
   if(PollControlKey()) fR -= fDt;
   if(fTheta < 1.0E-5f) fTheta = 1.0E-5f;
   if(fTheta > XM_PI-1.0E-5f) fTheta = XM_PI-1.0E-5f;

   XMFLOAT3 fP;
   GetCameraTarget(nullptr, fP);
   fP.y += fR;
   SetCameraTarget(nullptr, fP);
   SetCameraPosition(nullptr, 5.0f, fTheta, fPhi);
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   SetDisplayMode(1024, 768);
   auto hCamera = CloneCamera(GetCameraCurrent());
   SetCameraFarPlane(hCamera, 1);
   SetCameraRadialDistance(nullptr, 4);
   SetCameraIsometricView(nullptr);

   XMFLOAT3 fP(1.5, 0, 1.5), fD(-1, 0, -1);
   SetLightType(nullptr, 3);
   SetLightSize(nullptr, XMFLOAT2(1, 1), 2);
   SetLightColor(nullptr, XMFLOAT3(1, 0.75, 0.25), 0.25);
   SetLightPosition(nullptr, fP);
   SetLightDirection(nullptr, fD);

   // Enter the main loop
   while(Looping())
   {
      ClearDisplay();
      UpdateCamera();
      DrawWorldTriad(1.75);
      TurnCamera(hCamera, XMFLOAT3(1, 0, 0), 0.005f);
      DrawCameraFrustum(hCamera, 0xFF00FFFF, 0.25);
      TurnLight(nullptr, fP, fD, 0.005f);
      DrawLightGeometry(nullptr, 0.75, 3, 0.25, 0.125);
      PrintText(10, 40, L"GetCameraViewWidth = \f#ff00ff%f", GetCameraViewWidth(nullptr));
      PrintText(10, 70, L"GetCameraViewAngle = \f#ff00ff%f", GetCameraViewAngle(nullptr));
      PrintText(10, 100, L"GetCameraNearPlane = \f#ff00ff%f", GetCameraNearPlane(nullptr));
      PrintText(10, 130, L"GetCameraFarPlane = \f#ff00ff%f", GetCameraFarPlane(nullptr));
      PrintText(10, 160, L"GetCameraRadialDistance = \f#ff00ff%.3f", GetCameraRadialDistance(nullptr));
      PrintText(10, 190, L"GetCameraInclineAngle = \f#ff00ff%.3f degrees", GetCameraInclineAngle(nullptr)*180/XM_PI);
      PrintText(10, 220, L"GetCameraAzimuthAngle = \f#ff00ff%.3f degrees", GetCameraAzimuthAngle(nullptr)*180/XM_PI);
      UpdateDisplay(60);
   }
}
