#include "axbreeze.h"
HANDLE hPlot0;
bool bZoom = 0;

void CameraPosition()
{
   short sDx = 0, sDy = 0, sDz = 0;
   const float fDs = 0.002f;
   float fTheta = 0.0f, fPhi = 0.0f;
   GetMouseMove(MK_LBUTTON | MK_MBUTTON | MK_RBUTTON, sDx, sDy, sDz);
   fTheta -= fDs*sDy;
   fPhi -= fDs*sDx;
   PitchCamera(nullptr, fTheta);
   RollCamera(nullptr, fPhi);
   MoveCamera(nullptr, fDs*sDz);
}

void UpdateCamera()
{
   const float fDt = 0.1f;
   float fDx = 0, fDy = 0, fDz = 0;
   XMFLOAT3 fD(0, 0, 0);

   if(PollUpKey()) fDy -= fDt;
   if(PollDownKey()) fDy += fDt;
   if(PollLeftKey()) fDx += fDt;
   if(PollRightKey()) fDx -= fDt;
   if(PollShiftKey()) fDz += 0.25f*fDt;
   if(PollControlKey()) fDz -= 0.25f*fDt;
   if(PollVirtualKey(VK_W)) fD.y -= fDt;
   if(PollVirtualKey(VK_S)) fD.y += fDt;
   if(PollVirtualKey(VK_A)) fD.x += fDt;
   if(PollVirtualKey(VK_D)) fD.x -= fDt;
   if(PollVirtualKey(VK_Q)) fD.z += 0.25f*fDt;
   if(PollVirtualKey(VK_E)) fD.z -= 0.25f*fDt;

   ShiftCamera(nullptr, XMFLOAT3(fDx, fDy, fDz));
   PanCamera(nullptr, fD);
   AXRECT aRect(300, 300, 500, 400);
   DrawRectangle(aRect);
   if(PollSpaceKey() && !bZoom)
   {
      bZoom = 1;
      SetCameraZoomView(nullptr, aRect);
   }
   if(PollControlKey() && !bZoom)
   {
      bZoom = 1;
      GetBoundingRectangle(hPlot0, aRect);
      SetCameraZoomView(nullptr, aRect);
   }
   else if(PollReturnKey())
   {
      bZoom = 0;
      SetCameraZoomView(nullptr, AXRECT(-20, -20, GetDisplayClientSize().w+20, GetDisplayClientSize().h+20));
   }
}

void UpdateScene()
{
   CameraPosition();
   UpdateCamera();
   ClearDisplay(0);
   PrintText(8, 6, L"\f#ffff00FPS: \f#ff00ff%d", GetDisplayFrameRate());
   SetGraphLegend(L"Pressure", 0, 5, 0, false);
   DrawGraph(hPlot0, true, true, true);
   DrawGraphVolume(hPlot0, 0xffff00ff);
   DrawTriad(XMFLOAT3(-0.125, -0.125, -0.125), XMFLOAT3(1, 0, 0), XMFLOAT3(0, 1, 0), 2, 0.25, true);
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   WORD wWidth = 1024, wHeight = 768;

   // Initialize graphics, input, and audio
   SetDisplayMode(wWidth, wHeight);
   SetLightType(nullptr, 2);
   SetLightPosition(nullptr, XMFLOAT3(-5, -5, 5));
   SetCameraTarget(nullptr, XMFLOAT3(10, 10, 2));
   SetCameraPosition(nullptr, XMFLOAT3(-5, -5, 10));
   SetCameraNearPlane(nullptr, 0.25);
   SetCameraViewWidth(nullptr, 2);

   hPlot0 = LoadGraph(XMFLOAT3(10, 10, 5), L"..\\Media\\Heightmap.jpg", 0);
   SetForeColor(0xffffff00);

   // Enter the game loop
   while(Looping())
   {
      UpdateScene();
      XMFLOAT3 fP, fT, fU;
      GetCameraPosition(nullptr, fP);
      GetCameraTarget(nullptr, fT);
      GetCameraUpvector(nullptr, fU);
      float fW = GetCameraViewWidth(nullptr);
      short sY = 10;
      DrawText(250, sY, L"GetCameraPosition = %f, %f, %f", fP.x, fP.y, fP.z);
      sY += 30;
      DrawText(250, sY, L"GetCameraTarget = %f, %f, %f", fT.x, fT.y, fT.z);
      sY += 30;
      DrawText(250, sY, L"GetCameraUpvector = %f, %f, %f", fU.x, fU.y, fU.z);
      sY += 30;
      DrawText(250, sY, L"GetCameraViewWidth = %f", fW);
      XMMATRIX mV, mP;
      GetCameraView(nullptr, mV);
      GetCameraProjection(nullptr, mP);
      XMFLOAT4X4 fM;
      XMStoreFloat4x4(&fM, mV);
      sY += 30;
      DrawText(200, sY, L"mV[0] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[0][0], fM.m[0][1], fM.m[0][2], fM.m[0][3]);
      sY += 30;
      DrawText(200, sY, L"mV[1] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[1][0], fM.m[1][1], fM.m[1][2], fM.m[1][3]);
      sY += 30;
      DrawText(200, sY, L"mV[2] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[2][0], fM.m[2][1], fM.m[2][2], fM.m[2][3]);
      sY += 30;
      DrawText(200, sY, L"mV[3] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[3][0], fM.m[3][1], fM.m[3][2], fM.m[3][3]);
      XMStoreFloat4x4(&fM, mP);
      sY -= 90;
      DrawText(620, sY, L"mP[0] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[0][0], fM.m[0][1], fM.m[0][2], fM.m[0][3]);
      sY += 30;
      DrawText(620, sY, L"mP[1] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[1][0], fM.m[1][1], fM.m[1][2], fM.m[1][3]);
      sY += 30;
      DrawText(620, sY, L"mP[2] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[2][0], fM.m[2][1], fM.m[2][2], fM.m[2][3]);
      sY += 30;
      DrawText(620, sY, L"mP[3] = %0.2f, %0.2f, %0.2f, %0.2f", fM.m[3][0], fM.m[3][1], fM.m[3][2], fM.m[3][3]);
      DrawText(200, 700, L"Press Arrow/Control/Shift/W/A/S/D key to move camera");
      DrawText(200, 730, L"Press SPACE bar to zoom in, ENTER to reset zoom zrea");
      UpdateDisplay(60);
   }
}
