#include "axbreeze.h"
#define MAX_BUFFER 100*1024 // 100K buffer size

XMMATRIX mWorld0;
XMMATRIX mWorld1;
HANDLE hImage0;
HANDLE hObject0;
HANDLE hObject1;
HANDLE hSkybox0;
HANDLE hTerrain0;

void UpdateScene()
{
   float t = GetGameTime();

   XMMATRIX mTranslate;
   XMMATRIX mOrbit;
   XMMATRIX mSpin;
   XMMATRIX mScale;
   XMMATRIX mView;
   XMMATRIX mProj;

   ClearDisplay();

   mWorld0 = XMMatrixRotationZ(-0.5f*t);
   mScale = XMMatrixScaling(0.5f, 0.5f, 0.5f);
   mTranslate = XMMatrixTranslation(-1, 0.5f, 8);
   mWorld0 = XMMatrixMultiply(XMMatrixMultiply(mWorld0, mScale), mTranslate);
   SetObjectTransform(hObject0, mWorld0);

   mSpin = XMMatrixRotationZ(0);
   mOrbit = XMMatrixRotationZ(-0.5f*t);
   mTranslate = XMMatrixTranslation(16, 0.5f, 8);
   mScale = XMMatrixScaling(0.4f, 0.4f, 0.4f);
   mWorld1 = XMMatrixMultiply(XMMatrixMultiply(XMMatrixMultiply(mScale, mSpin), mTranslate), mOrbit);
   SetObjectTransform(hObject1, mWorld1);

   DrawSprite();
   DrawObject(hObject0);
   DrawObject(hObject1);
   DrawTerrain(hTerrain0);
   DrawSkybox(hSkybox0);

   auto fP = XMFLOAT3(0, 4, 5);
   auto fR = XMFLOAT3(-0.125, 0, 0);
   SetMirrorReflection(0.5);
   DrawMirrorImage(hImage0, fP, fR, 8);
   DrawSprite();
   DrawObject(hObject0);
   DrawObject(hObject1);
   DrawTerrain(hTerrain0);
   DrawSkybox(hSkybox0);
   SetMirrorReflection(0);
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   WORD wWidth = 1024, wHeight = 768;

   // Initialize graphics, input, and audio
   SetDisplayMode(wWidth, wHeight);

   // Load "helloworld.jpg" to image number 0
   hImage0 = LoadImage(L"..\\Image\\helloworld.jpg");
   BYTE cBuffer[6][MAX_BUFFER]; // Make sure file sizes are smaller than MAX_BUFFER!
   WCHAR* pSource[6] = { L"..\\Image\\skybox10_u.jpg", L"..\\Image\\skybox11_l.jpg",
                         L"..\\Image\\skybox12_f.jpg", L"..\\Image\\skybox13_r.jpg",
                         L"..\\Image\\skybox14_b.jpg", L"..\\Image\\skybox15_d.jpg" };

   BYTE *pRfm[6];
   int iRfm[6];
   for(UINT i = 0; i < 6; i++)
   {
      pRfm[i] = cBuffer[i];
      iRfm[i] = LoadFile(cBuffer[i], MAX_BUFFER, pSource[i]);
   }
   hSkybox0 = LoadSkybox(XMFLOAT3(500, 500, 500), pRfm, iRfm);
   HANDLE hSprite0 = LoadSprite(L"..\\Image\\audrey_kitty.gif");
   SetSprite(hSprite0, 4, 2, 0.1f);
   SetSpriteScale(hSprite0, XMFLOAT2(12.5f, 12.5f));
   SetSpritePosition(hSprite0, XMFLOAT3(2, -4, 4));
   hTerrain0 = LoadTerrain(XMFLOAT3(40, 40, 6), L"..\\Media\\Heightmap.jpg");
   WCHAR* pTerrain[5] = { L"..\\Media\\snow.png", L"..\\Media\\dirt.png", L"..\\Media\\stone.png",
                          L"..\\Media\\grass.png", L"..\\Media\\blend.jpg" };
   SetTerrainTexture(hTerrain0, pTerrain);
   SetTerrainTiling(hTerrain0, XMFLOAT2(3, 3));
   SetTerrainPosition(hTerrain0, XMFLOAT3(-20, -20, 2));

   hObject0 = LoadObject(L"..\\Model\\LilyPT0.3ds", 0.2f);
   hObject1 = LoadObject(L"..\\Model\\heart.3ds", 0.2f);

   SetCameraPosition(nullptr, XMFLOAT3(0, -20, 10));

   // Enter the game loop
   while(Looping())
   {
      UpdateScene();
      TurnCamera(nullptr, XMFLOAT3(0, 0, 1), 0.0025f);
      UpdateDisplay(100);
   }
}
