#include "axbreeze.h"
void DrawWorldTriad(float fSize);

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   SetDisplayMode(1024, 768);
   SetCameraRadialDistance(nullptr, 5);

   WCHAR* pDefault = L"Default view (hLight[\f#00ff00%d\f~#])";
   WCHAR* pView = pDefault;
   XMFLOAT3 fPosition[3], fDirection[3], fUpvector[3];
   UINT uLight = 0;
   HANDLE hLight[3];
   hLight[0] = GetLightCurrent();
   hLight[1] = CreateLight(XMFLOAT3(-1, 0, 0), XMFLOAT3(1, 0, 0), XMFLOAT2(1, 0.5), 0.25);
   hLight[2] = CloneLight(hLight[0]);
   SetLightType(hLight[1], 2);
   SetLightType(hLight[2], 3, 8);
   SetLightSize(hLight[0], XMFLOAT2(0.75, 1), 0.5);
   SetLightSize(hLight[2], XMFLOAT2(0.75, 0.5), 0.5);
   SetLightColor(hLight[0], XMFLOAT3(1, 1, 0), 0.25);
   SetLightColor(hLight[1], XMFLOAT3(0, 1, 1), 0.25);
   SetLightColor(hLight[2], XMFLOAT3(1, 0, 1), 0.25);
   SetLightPosition(hLight[0], XMFLOAT3(2, 0, 0), XMFLOAT3(-1, 0, 0), XMFLOAT3(0, 0, 1));
   SetLightPosition(hLight[1], XMFLOAT3(-2, 0, 0));
   SetLightPosition(hLight[2], XMFLOAT3(0, 0, 2));
   SetLightDirection(hLight[1], XMFLOAT3(1, 0, 0));
   SetLightTarget(hLight[2], XMFLOAT3(0, 0, 0));
   for(UINT i = 0; i < 3; i++)
   {
      GetLightPosition(hLight[i], fPosition[i]);
      GetLightDirection(hLight[i], fDirection[i]);
      GetLightUpvector(hLight[i], fUpvector[i]);
   }

   // Enter the main loop
   while(Looping())
   {
      ClearDisplay();
      DrawText(10, 10, L"Press 0 to reset light view (Arrow/Control/Shift/W/A/S/D key to move/rotate)");
      DrawText(10, 40, L"Space bar to operate light 0-2 (Enter key to set current light 0-2)");
      BYTE oKey = GetVirtualKey();
      switch(oKey)
      {
         case VK_0:
         {
            for(UINT i = 0; i < 3; i++) SetLightPosition(hLight[i], fPosition[i], fDirection[i], fUpvector[i]);
            pView = pDefault;
            break;
         }
         case VK_UP:
         {
            TurnLight(hLight[uLight], XMFLOAT3(0, 0, 0), XMFLOAT3(1, 0, 0), -0.03125);
            pView = L"TurnLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0), XMFLOAT3(1, 0, 0), -0.03125)";
            break;
         }
         case VK_DOWN:
         {
            TurnLight(hLight[uLight], XMFLOAT3(0, 0, 0), XMFLOAT3(1, 0, 0), 0.03125);
            pView = L"TurnLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0), XMFLOAT3(1, 0, 0), 0.03125)";
            break;
         }
         case VK_LCONTROL:
         {
            TurnLight(hLight[uLight], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 1, 0), -0.03125);
            pView = L"TurnLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 1, 0), -0.03125)";
            break;
         }
         case VK_RCONTROL:
         {
            TurnLight(hLight[uLight], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 1, 0), 0.03125);
            pView = L"TurnLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 1, 0), 0.03125)";
            break;
         }
         case VK_LEFT:
         {
            TurnLight(hLight[uLight], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 0, 1), -0.03125);
            pView = L"TurnLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 0, 1), -0.03125)";
            break;
         }
         case VK_RIGHT:
         {
            TurnLight(hLight[uLight], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 0, 1), 0.03125);
            pView = L"TurnLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0), XMFLOAT3(0, 0, 1), 0.03125)";
            break;
         }
         case VK_W:
         {
            ShiftLight(hLight[uLight], XMFLOAT3(0, 0, 0.03125));
            pView = L"ShiftLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, 0.03125))";
            break;
         }
         case VK_S:
         {
            ShiftLight(hLight[uLight], XMFLOAT3(0, 0, -0.03125));
            pView = L"ShiftLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0, -0.03125))";
            break;
         }
         case VK_LSHIFT:
         {
            ShiftLight(hLight[uLight], XMFLOAT3(0, -0.03125, 0));
            pView = L"ShiftLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, -0.03125, 0))";
            break;
         }
         case VK_RSHIFT:
         {
            ShiftLight(hLight[uLight], XMFLOAT3(0, 0.03125, 0));
            pView = L"ShiftLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0, 0.03125, 0))";
            break;
         }
         case VK_A:
         {
            ShiftLight(hLight[uLight], XMFLOAT3(-0.03125, 0, 0));
            pView = L"ShiftLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(-0.03125, 0, 0))";
            break;
         }
         case VK_D:
         {
            ShiftLight(hLight[uLight], XMFLOAT3(0.03125, 0, 0));
            pView = L"ShiftLight\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff], XMFLOAT3(0.03125, 0, 0))";
            break;
         }
         case VK_SPACE:
         {
            uLight = ++uLight%3;
            pView = L"Controlled Light = hLight[\f#00ff00%d\f~#]";
            break;
         }
         case VK_RETURN:
         {
            SetLightCurrent(hLight[uLight]);
            pView = L"SetLightCurrent\f#ff00ff(hLight[\f#00ff00%d\f#ff00ff])";
            break;
         }
      }
      for(UINT i = 0; i < 3; i++)
      {
         if(hLight[i] == GetLightCurrent())
         {
            PrintText(10, 70, L"GetLightCurrent = hLight[\f#00ff00%d\f~#]", i);
         }
         DrawLightGeometry(hLight[i], 0.75, 3, 1);
      }
      DrawWorldTriad(0.75);
      XMFLOAT3 fH, fD, fU;
      GetLightHorizon(hLight[uLight], fH);
      GetLightDirection(hLight[uLight], fD);
      GetLightUpvector(hLight[uLight], fU);
      PrintText(10, 620, L"GetLightHorizon = \f#ff00ff%.1f, %.1f, %.1f", fH.x, fH.y, fH.z);
      PrintText(10, 650, L"GetLightDirection = \f#ff00ff%.1f, %.1f, %.1f", fD.x, fD.y, fD.z);
      PrintText(10, 680, L"GetLightUpvector = \f#ff00ff%.1f, %.1f, %.1f", fU.x, fU.y, fU.z);
      PrintText(512, 740, 0, 0, pView, uLight);
      UpdateDisplay(30);
      if(oKey == VK_SPACE || oKey == VK_RETURN) while(PollVirtualKey(oKey));
   }
}
