#include "axbreeze.h"
#include "resource.h"
#define MAX_BUFFER 900*1024 // 900K buffer size
void UpdateCameraView(); // Manipulate camera view

XMFLOAT3 fC;
XMFLOAT3 fU;
XMFLOAT3 fT;
XMFLOAT3 fP;
HANDLE hSkybox0;
XMMATRIX mWorld1;
XMMATRIX mWorld2;
const UINT nVertex = 101;
const UINT nIndex = 3*(nVertex-1);
XMFLOAT3 fV[nVertex] = { XMFLOAT3(0.0f, -2.0f, 0.0f) };
XMFLOAT3 fW[nVertex]; // Vertices in world space
UINT uColor[nVertex] = { 0xFF0000FF };
UINT uIndex[nIndex];
float fRadius = 2.0f, fHeight = 3.0f, fTheta;

void UpdateScene()
{
   float t = GetGameTime();

   XMMATRIX mTranslate;
   XMMATRIX mOrbit;
   XMMATRIX mSpin;
   XMMATRIX mScale;

   mWorld1 = XMMatrixRotationZ(-0.5f*t);
   mScale = XMMatrixScaling(0.5f, 0.5f, 0.5f);
   mTranslate = XMMatrixTranslation(-1.0f, 10.0f, 0.5f);
   mWorld1 = XMMatrixMultiply(XMMatrixMultiply(mWorld1, mScale), mTranslate);

   mSpin = XMMatrixRotationZ(0.0f);
   mOrbit = XMMatrixRotationZ(-0.5f*t);
   mTranslate = XMMatrixTranslation(16.0f, 5.0f, 0.5f);
   mScale = XMMatrixScaling(0.4f, 0.4f, 0.4f);
   mWorld2 = XMMatrixMultiply(XMMatrixMultiply(XMMatrixMultiply(mScale, mSpin), mTranslate), mOrbit);

   ClearDisplay(0xFF00FF00);

   GetCameraPosition(nullptr, fC);
   GetCameraDirection(nullptr, fP);
   GetCameraUpvector(nullptr, fU);
   GetCameraTarget(nullptr, fT);

   XMVector3TransformCoordStream(fW, sizeof(XMFLOAT3), fV, sizeof(XMFLOAT3), nVertex, mWorld1); // Transform to world space
   DrawTriangle(fW, uColor, nVertex, uIndex, nIndex);
   XMVector3TransformCoordStream(fW, sizeof(XMFLOAT3), fV, sizeof(XMFLOAT3), nVertex, mWorld2); // Transform to world space
   DrawTriangle(fW, uColor, nVertex, uIndex, nIndex);
   DrawSprite();
   DrawSkybox(hSkybox0);
   SetForeColor(0xFFFFFF00);
   DrawText(100, 60, L"Camera rotation and movement, use arrow, shift, and control keys");
   DrawText(100, 660, L"%f", GetCameraRadialDistance(nullptr));
   SetForeColor(0xFFFF0000);
   DrawText(100, 100, L"%f", fC.x);
   DrawText(100, 140, L"%f", fC.y);
   DrawText(100, 180, L"%f", fC.z);
   SetForeColor(0xFF00FF00);
   DrawText(300, 100, L"%f", fP.x);
   DrawText(300, 140, L"%f", fP.y);
   DrawText(300, 180, L"%f", fP.z);
   SetForeColor(0xFFFF00FF);
   DrawText(500, 100, L"%f", fU.x);
   DrawText(500, 140, L"%f", fU.y);
   DrawText(500, 180, L"%f", fU.z);
   SetForeColor(0xFF0000FF);
   DrawText(700, 100, L"%f", fT.x);
   DrawText(700, 140, L"%f", fT.y);
   DrawText(700, 180, L"%f", fT.z);
   short sY = 415;
   DrawText(15, sY, L"Display window size");
   DrawText(300, sY, L"%d", GetDisplayWindowSize().w);
   DrawText(400, sY, L"%d", GetDisplayWindowSize().h);
   SetForeColor(0xFFFFFF00);
   sY += 40;
   DrawText(15, sY, L"Display client size");
   DrawText(300, sY, L"%d", GetDisplayClientSize().w);
   DrawText(400, sY, L"%d", GetDisplayClientSize().h);
   sY += 40;
   DrawText(15, sY, L"Display screen size");
   DrawText(300, sY, L"%d", GetDisplayScreenSize().w);
   DrawText(400, sY, L"%d", GetDisplayScreenSize().h);
   SetForeColor(0xFF00FF00);
   sY += 40;
   DrawText(15, sY, L"Display refresh rate %u Hz", GetDisplayRefreshRate());
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   SetDisplayMode(GetMonitorSize().w, GetMonitorSize().h, 4);

   WCHAR* pSource[6] = { L"..\\Image\\skybox00_u.jpg", L"..\\Image\\skybox01_l.jpg",
                         L"..\\Image\\skybox02_f.jpg", L"..\\Image\\skybox03_r.jpg",
                         L"..\\Image\\skybox04_b.jpg", L"..\\Image\\skybox05_d.jpg" };
   hSkybox0 = LoadSkybox(XMFLOAT3(500.0f, 500.0f, 500.0f), pSource);

   for(UINT i = 1; i < nVertex; i++)
   {
      fTheta = 2*XM_PI*(i-1)/(nVertex-2);
      fV[i] = XMFLOAT3(fRadius*cos(fTheta), fV[0].y+fHeight, fRadius*sin(fTheta));
      uColor[i] = 0xFF000F0F + (i<<16) + ((2*i)<<8) + i;
      uIndex[3*(i-1)] = 0;
      uIndex[3*(i-1)+1] = i;
      uIndex[3*(i-1)+2] = i < nVertex-1 ? i+1 : 1;
   }

   HANDLE hSprite6 = LoadSprite(L"..\\Image\\Bear_walks_2.png", 0);
   HANDLE hSprite7 = LoadSprite(L"..\\Image\\Green_bird.png", 0);
   HANDLE hSprite8 = LoadSprite(L"..\\Image\\Cave_man.png", 0);
   SetSprite(hSprite6, 3, 2, 0.1f);
   SetSpritePosition(hSprite6, XMFLOAT3(-1.6f, 1.6f, -0.28f));
   SetSpriteVelocity(hSprite6, XMFLOAT3(0.06f, 0.0f, -0.03f), XMFLOAT3(0.01f, 0.01f, 0.0f));
   SetSpritePosition(hSprite7, XMFLOAT3(-1.8f, 1.25f, 0.2f));
   SetSpritePosition(hSprite8, XMFLOAT3(1.8f, 1.65f, -0.18f));
   HANDLE hSound6 = LoadSound(L"..\\Sound\\bear.wav");
   PlaySound(hSound6, 1);
   SetSprite(hSprite7, 4, 2, 0.03f);
   SetSpriteVelocity(hSprite7, XMFLOAT3(0.08f, 0.03f, 0.0f), XMFLOAT3(0.01f, 0.0f, 0.01f));
   SetSpriteAcceleration(hSprite7, XMFLOAT3(0.01f, 0.0f, 0.01f), XMFLOAT3(0.001f, 0.001f, 0.0f));
   HANDLE hSound7 = LoadSound(L"..\\Sound\\goose.wav");
   PlaySound(hSound7);
   SetSprite(hSprite8, 3, 3, 0.05f);
   SetSpriteVelocity(hSprite8, XMFLOAT3(-0.05f, 0.0f, -0.03f), XMFLOAT3(0.01f, 0.01f, 0.0f));

   EnableVerticalSync(1);

   // Enter the main loop
   while(Looping())
   {
      UpdateScene();
      UpdateCameraView();
      UpdateDisplay();
   }
}
