#include "axbreeze.h"
#include "resource.h"
#define MAX_BUFFER 100*1024 // 100K buffer size
void UpdateCameraView(); // Manipulate camera view

XMFLOAT3 fC;
XMFLOAT3 fU;
XMFLOAT3 fT;
XMFLOAT3 fP;
XMMATRIX mWorld1;
XMMATRIX mWorld2;

void UpdateScene()
{
   static UINT i = 0;
   static float t = 0.0f;
   if(++i%2) t = GetGameTime();

   XMMATRIX mTranslate;
   XMMATRIX mOrbit;
   XMMATRIX mSpin;
   XMMATRIX mScale;

   // 1st Transformation: Rotate around the origin
   mWorld1 = XMMatrixRotationRollPitchYaw(0, 0.0f, -t);
   mScale = XMMatrixScaling(0.5f, 0.5f, 0.5f);
   mWorld1 = XMMatrixMultiply(mWorld1, mScale);

   // 2nd Transformation: Orbit around the origin
   mSpin = XMMatrixRotationRollPitchYaw(0.25, 0.0f, t);
   mOrbit = XMMatrixRotationZ(-0.8f*t);
   mTranslate = XMMatrixTranslation(-2.0f, 0.0f, 0.0f);
   mScale = XMMatrixScaling(0.3f, 0.3f, 0.3f);
   mWorld2 = XMMatrixMultiply(XMMatrixMultiply(XMMatrixMultiply(mScale, mSpin), mTranslate), mOrbit);

   GetCameraPosition(nullptr, fC);
   GetCameraDirection(nullptr, fP);
   GetCameraUpvector(nullptr, fU);
   GetCameraTarget(nullptr, fT);

   UpdateCameraView();
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   SetDisplayMode(560, 420);
   SetTextSize(16);

   HANDLE hObject1 = LoadObject(L"..\\Model\\RoseRT0.3ds", 0.15f);
   HANDLE hObject2 = LoadObject(L"..\\Model\\LilyPT0.3ds", 0.15f);

   SetCameraPosition(nullptr, XMFLOAT3(0.0f, -10.0f, 0.0f), XMFLOAT3(0.0f, 0.0f, 1.0f));
   HANDLE hCamera0 = GetCameraCurrent();
   HANDLE hCamera1 = CreateCamera(2.0f, 0.01f, 10000.0f);
   SetCameraCurrent(hCamera1);
   SetCameraPosition(nullptr, XMFLOAT3(0.0f, 0.0f, 8.0f), XMFLOAT3(0.0f, 1.0f, 0.0f));

   BYTE cBuffer[6][MAX_BUFFER]; // Make sure file sizes are smaller than MAX_BUFFER!
   WCHAR* pSource[6] = { L"..\\Image\\skybox10_u.jpg", L"..\\Image\\skybox11_l.jpg",
                         L"..\\Image\\skybox12_f.jpg", L"..\\Image\\skybox13_r.jpg",
                         L"..\\Image\\skybox14_b.jpg", L"..\\Image\\skybox15_d.jpg" };

   BYTE *pRfm[6];
   int iRfm[6];
   for(short i = 0; i < 6; i++)
   {
      pRfm[i] = cBuffer[i];
      iRfm[i] = LoadFile(cBuffer[i], MAX_BUFFER, pSource[i]);
   }
   HANDLE hSkybox = LoadSkybox(XMFLOAT3(90.0f, 90.0f, 90.0f), pRfm, iRfm);

   SpeakText(L"Welcome to use <pitch middle = '+10'/> Axbreeze");

   // Enter the main loop
   while(Looping())
   {
      ClearDisplay(0xff7f7f7f);
      SetCameraCurrent(hCamera0);
      SetDisplayViewport(AXRECT(0, 0, 275, 420));
      UpdateScene();
      SetObjectTransform(hObject1, mWorld1);
      DrawObject(hObject1);
      SetObjectTransform(hObject2, mWorld2);
      DrawObject(hObject2);
      DrawSkybox(hSkybox);
      SetForeColor(0xFFFF0000);
      DrawText(50, 50, L"%f", fC.x);
      DrawText(50, 70, L"%f", fC.y);
      DrawText(50, 90, L"%f", fC.z);
      SetForeColor(0xFF00FF00);
      DrawText(150, 50, L"%f", fU.x);
      DrawText(150, 70, L"%f", fU.y);
      DrawText(150, 90, L"%f", fU.z);
      SetCameraCurrent(hCamera1);
      SetDisplayViewport(AXRECT(285, 0, 275, 420));
      UpdateScene();
      SetObjectTransform(hObject1, mWorld1);
      DrawObject(hObject1);
      SetObjectTransform(hObject2, mWorld2);
      DrawObject(hObject2);
      DrawSkybox(hSkybox);
      SetForeColor(0xFFFF0000);
      DrawText(335, 50, L"%f", fC.x);
      DrawText(335, 70, L"%f", fC.y);
      DrawText(335, 90, L"%f", fC.z);
      SetForeColor(0xFF00FF00);
      DrawText(425, 50, L"%f", fT.x);
      DrawText(425, 70, L"%f", fT.y);
      DrawText(425, 90, L"%f", fT.z);
      PrintText(50, 25, L"\f#ffff00Camera \f#ff00000 \f#ff00ff(Side View)");
      PrintText(335, 25, L"\f#ffff00Camera \f#00ff001 \f#ff00ff(Top View)");
      UpdateDisplay(0);
   }
}
