#include "axbreeze.h"

WCHAR* ppFile[];

// UI_DEFAULT  = 0x80
// UI_GROUPED  = 0x40
// UI_CALLBACK = 0x20
// UI_COLORED  = 0x10
// UI_CHECKED  = 0x08
// UI_BITMAP   = 0x04
// UI_SELECT   = 0x02
// UI_SUBLEVEL = 0x01

// State hexadecimal number meaning (for menu item callback)
// UI_DEFAULT  = Specify that this is the default menu item for the specified menu
// UI_GROUPED  = Check menu radio item (output only, skip all other flag options)
// UI_CALLBACK = Mouse left button clicked (input only)
// UI_COLORED  = Disable the menu item and gray it (group selection if UI_SUBLEVEL is set)
// UI_CHECKED  = Place a check mark next to the menu item (group selection if UI_SUBLEVEL is set)
// UI_SUBLEVEL = Specify that group selection is set for UI_COLORED or UI_CHECKED

void MenuCallbackNew(AXMITE& aMite)
{
   aMite.m ^= UI_CHECKED;
   if(aMite.m & UI_CHECKED)
   {
      SpeakText(L"New menu checked!");
      EnablePopupMenuItem(ppFile[0], 3, 5, 0);
   }
   else
   {
      SpeakText(L"New menu unchecked!");
      EnablePopupMenuItem(ppFile[0], 3, 5, 1);
   }
}

void MenuCallbackOpen(AXMITE& aMite)
{
   if(aMite.m & UI_CALLBACK)
   {
      SpeakText(L"Mouse left button clicked!");
   }

   if(aMite.m & UI_COLORED)
   {
      aMite.m &= ~UI_COLORED;
      SpeakText(L"Open menu enabled!");
   }
   else if(!(aMite.m & UI_COLORED))
   {
      aMite.m |= UI_COLORED;
      SpeakText(L"Open menu grayed!");
   }
}

void MenuCallbackClose(AXMITE& aMite)
{
   aMite.m ^= UI_CHECKED;
   if(aMite.m & UI_CHECKED)
   {
      SpeakText(L"Close menu checked!");
   }
   else
   {
      SpeakText(L"Close menu unchecked!");
   }
}

void MenuCallbackSave(AXMITE&)
{
   SpeakText(L"Save menu selected!");
}

void MenuCallbackSaveAs(AXMITE&)
{
   SpeakText(L"Save As menu selected!");
}

void MenuCallbackSaveAll(AXMITE& aMite)
{
   aMite.m ^= UI_CHECKED;
   if(aMite.m & UI_CHECKED)
   {
      SpeakText(L"Save All menu checked!");
   }
   else
   {
      SpeakText(L"Save All menu unchecked!");
   }
}

void MenuCallbackRadioItem(AXMITE& aMite)
{
   if(aMite.m & UI_CALLBACK) aMite.m |= UI_DEFAULT | UI_GROUPED;
   SpeakText(L"Radio item %d!", aMite.i);
}

void MenuCallbackGroupSelection(AXMITE& aMite)
{
   aMite.m |= UI_SUBLEVEL;
   aMite.m ^= UI_COLORED;
   SpeakText(L"Group selection %d!", aMite.i);
}

void MenuCallbackSecondLevel(AXMITE& aMite)
{
   if(aMite.m & UI_COLORED)
   {
      aMite.m &= ~UI_COLORED;
      SpeakText(L"Second level menu enabled!");
   }
}

void MenuCallbackSubmenu(AXMITE&)
{
   SpeakText(L"Submenu item selected!");
}

void MenuCallbackExit(AXMITE&)
{
   SpeakText(L"Exit menu selected!");
   DestroyDisplay();
}

// State hexadecimal number meaning (for toolbar button callback)
// UI_DEFAULT  = Specify that group selection is for all button items (output only)
// UI_GROUPED  = Check button radio item (output only, skip all other flag options)
// UI_CALLBACK = Mouse left button clicked (input only)
// UI_COLORED  = Disable the button item and gray it (group selection if UI_SUBLEVEL is set)
// UI_CHECKED  = Check the specified button item in toolbar (group selection if UI_SUBLEVEL is set)
// UI_SUBLEVEL = Specify that group selection is set for UI_COLORED or UI_CHECKED

void ButtonCallbackNew(AXMITE& aMite)
{
   aMite.t ^= UI_CHECKED;
   aMite.t |= UI_SUBLEVEL;
   if(aMite.t & UI_CHECKED)
   {
      SpeakText(L"New button checked!");
   }
   else
   {
      SpeakText(L"New button unchecked!");
   }
}

void ButtonCallbackOpen(AXMITE& aMite)
{
   aMite.t |= UI_GROUPED;
   if(aMite.t & UI_CALLBACK)
   {
      SpeakText(L"Mouse left button clicked!");
   }

   if(aMite.t & UI_COLORED)
   {
      aMite.t &= ~UI_COLORED;
      SpeakText(L"Open button enabled!");
   }
   else if(!(aMite.t & UI_COLORED))
   {
      aMite.t |= UI_COLORED;
      SpeakText(L"Open button grayed!");
   }
}

void ButtonCallbackClose(AXMITE& aMite)
{
   aMite.t ^= UI_CHECKED;
   if(aMite.t & UI_CHECKED)
   {
      SpeakText(L"Close button checked!");
   }
   else
   {
      SpeakText(L"Close button unchecked!");
   }
}

void ButtonCallbackUndo(AXMITE& aMite)
{
   aMite.t ^= UI_CHECKED;
   aMite.t |= UI_DEFAULT | UI_GROUPED;
   if(aMite.t & UI_CHECKED)
   {
      SpeakText(L"Undo button checked!");
   }
   else
   {
      SpeakText(L"Undo button unchecked!");
   }
}

void ButtonCallbackMenu(AXMITE& aMite)
{
   aMite.t |= UI_GROUPED | UI_CHECKED;
   SpeakText(L"Button callback menu selected!");
}

void ButtonCallbackRedo(AXMITE& aMite)
{
   aMite.t |= UI_DEFAULT | UI_GROUPED;
   if(aMite.t & UI_CALLBACK)
   {
      SpeakText(L"Mouse left button clicked!");
   }

   if(aMite.t & UI_COLORED)
   {
      aMite.t &= ~UI_COLORED;
      SpeakText(L"Redo button enabled!");
   }
   else if(!(aMite.t & UI_COLORED))
   {
      aMite.t |= UI_COLORED;
      SpeakText(L"Redo button grayed!");
   }
}

void ButtonCallbackHome(AXMITE& aMite)
{
   aMite.t ^= UI_CHECKED;
   if(aMite.t & UI_CHECKED)
   {
      SpeakText(L"Home button checked!");
      ShellExecute(nullptr, L"open", L"http://www.nodapro.com", nullptr, nullptr, SW_SHOWNORMAL);
   }
   else
   {
      SpeakText(L"Home button unchecked!");
   }

   if(aMite.t & UI_COLORED)
   {
      aMite.t &= ~UI_COLORED;
      SpeakText(L"Home button enabled!");
   }
   else if(!(aMite.t & UI_COLORED))
   {
      aMite.t |= UI_COLORED;
      SpeakText(L"Home button grayed!");
   }
}

// Leading hexadecimal number meaning (for menu items only)
// UI_DEFAULT  = 0x80 Specify that this is the default menu item for the specified menu
// UI_GROUPED  = 0x40 Check the specified menu item and make it a radio item
// UI_CALLBACK = 0x20 Specify that a callback function is provided
// UI_COLORED  = 0x10 Disable the menu item and gray it so that it cannot be selected
// UI_CHECKED  = 0x08 Place a check mark next to the menu item
// UI_BITMAP   = 0x04 Specify that a custom unchecked bitmap is provided
// UI_SELECT   = 0x02 Specify that a custom checked bitmap is provided
// UI_SUBLEVEL = 0x01 Create a submenu associated with the menu item

WCHAR* ppFile[] = {
   L"&File",                          // Menu name (no leading hexadecimal number)
      L"20 &New\tCtrl+N",             // Callback function is provided
         (WCHAR*)MenuCallbackNew,     // Function address (no leading hexadecimal number)
      L"28 &Open\tCtrl+O",            // CHECKED
         (WCHAR*)MenuCallbackOpen,    // Function address (no leading hexadecimal number)
      L"24 Close",                    // Custom unchecked bitmap is provided
         (WCHAR*)MenuCallbackClose,   // Function address (no leading hexadecimal number)
         L"IMG_15_CLOSE",             // Button image resource name (defined in resource.rc)
      L"--",                          // SEPARATOR
      L"20 &Save\tCtrl+S",
         (WCHAR*)MenuCallbackSave,    // Function address (no leading hexadecimal number)
      L"20 Save As...\tAlt+Shift+S",
         (WCHAR*)MenuCallbackSaveAs,  // Function address (no leading hexadecimal number)
      L"26 Save All\tCtrl+Enter",
         (WCHAR*)MenuCallbackSaveAll, // Function address (no leading hexadecimal number)
         L"IMG_15_SAVEALL0",          // Custom unchecked bitmap resource name (defined in resource.rc)
         L"IMG_15_SAVEALL1",          // Custom checked bitmap resource name (defined in resource.rc)
      L"--",                          // SEPARATOR
      L"28 &Exit\tAlt+F4",
         (WCHAR*)MenuCallbackExit,    // Function address (no leading hexadecimal number)
   nullptr };                            // End of menu (nullptr)

WCHAR* ppEdit[] = {
   L"&Edit",                                    // Menu name (no leading hexadecimal number)
      L"10 &Undo\tCtrl+Z",                      // GRAYED
      L"10 &Redo\tCtrl+Y",                      // GRAYED
      L"--",                                    // SEPARATOR
      L"60 Cut\tCtrl+X",
         (WCHAR*)MenuCallbackRadioItem,
      L"60 Copy\tCtrl+C",
         (WCHAR*)MenuCallbackRadioItem,
      L"E0 Paste\tCtrl+V",
         (WCHAR*)MenuCallbackRadioItem,
      L"--",                                    // SEPARATOR
      L"01 Select",                             // Submenu name (with leading hexadecimal number)
         L"20 Select All\tCtrl+A",              // Submenu item
            (WCHAR*)MenuCallbackRadioItem,
         L"20 Select Line\tCtrl+E",             // Submenu item
            (WCHAR*)MenuCallbackRadioItem,
         L"20 Select Word\tCtrl+D",             // Submenu item
            (WCHAR*)MenuCallbackRadioItem,
         L"20 Select Block\tCtrl+B",            // Submenu item
            (WCHAR*)MenuCallbackRadioItem,
      nullptr,                                     // End of submenu (nullptr)
      L"09 Change Case",                        // Submenu name (with leading hexadecimal number)
         L"20 Upper Case\tCtrl+U",
            (WCHAR*)MenuCallbackRadioItem,
         L"20 Lower Case\tCtrl+L",
            (WCHAR*)MenuCallbackRadioItem,
         L"--",                                 // SEPARATOR
         L"2F Second Level",                    // Submenu name (second level)
            (WCHAR*)MenuCallbackSecondLevel,
            L"IMG_15_SAVEALL0",
            L"IMG_15_SAVEALL1",
            L"28 Submenu Item\tCtrl+F5",        // Submenu item (callback function is provided)
               (WCHAR*)MenuCallbackSubmenu,     // Function address (no leading hexadecimal number)
            L"20 Submenu Test\tF5",             // Submenu test (same callback function is used)
               (WCHAR*)MenuCallbackSubmenu,     // Function address (no leading hexadecimal number)
         nullptr,                                  // End of submenu (second level)
         L"20 Capitalize\tCtrl+Shift+U",
            (WCHAR*)MenuCallbackGroupSelection,
         L"20 Invert Case\tCtrl+Shift+L",
            (WCHAR*)MenuCallbackGroupSelection,
      nullptr,                                     // End of submenu (nullptr)
      L"40 Column Mode\tAlt+C",
   nullptr };                                      // End of menu (nullptr)

WCHAR* ppSearch[] = {
   L"&Search",                          // Menu name (no leading hexadecimal number)
      L"20 Find...\tCtrl+F",
         (WCHAR*)MenuCallbackRadioItem,
      L"20 Replace...\tCtrl+H",
         (WCHAR*)MenuCallbackRadioItem,
      L"--",                            // SEPARATOR
      L"20 Find Next\tF3",
         (WCHAR*)MenuCallbackRadioItem,
      L"20 Find Prev\tShift+F3",
         (WCHAR*)MenuCallbackRadioItem,
   nullptr };                              // End of menu (nullptr)

WCHAR* ppSearchPopup[] = {
   L"&Search",          // Menu name (no leading hexadecimal number)
      L"00 Find...",
      L"00 Replace...",
      L"--",            // SEPARATOR
      L"00 Find Next",
      L"00 Find Prev",
   nullptr };              // End of menu (nullptr)

// Leading hexadecimal number meaning (for toolbar buttons)
// UI_DEFAULT  = 0x80 Specify that the button automatically toggles between states
// UI_GROUPED  = 0x40 Add a popup menu associated with the button right mouse click
// UI_CALLBACK = 0x20 Specify that a callback function is provided
// UI_COLORED  = 0x10 Disable the button and gray it so that it cannot be clicked
// UI_CHECKED  = 0x08 Button is being clicked (pressed)
// UI_BITMAP   = 0x04 Add a bitmap to the button
// UI_SELECT   = 0x02 Specify that the bitmap is internal
// UI_SUBLEVEL = 0x01 Reserved

WCHAR* ppToolbar[] = {
   L"Toolbar",                       // Toolbar name (no leading hexadecimal number)
      L"26 New",                     // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackNew,  // Function address (no leading hexadecimal number)
         (WCHAR*)STD_FILENEW,        // Common control library image (defined in commctrl.h)
      L"26 Open",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackOpen, // Function address (no leading hexadecimal number)
         (WCHAR*)STD_FILEOPEN,
      L"06 Save",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_FILESAVE,
      L"--",                         // SEPARATOR
      L"06 Print",                   // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_PRINT,
      L"06 Print Preview",           // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_PRINTPRE,
      L"--",                         // SEPARATOR
      L"86 Cut",                     // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_CUT,
      L"86 Copy",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_COPY,
      L"86 Paste",                   // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_PASTE,
      L"--",                         // SEPARATOR
      L"6E Undo",                    // Button tooltip (popup menu for right mouse click)
         ppEdit[0],                  // Popup menu name
         (WCHAR*)ButtonCallbackUndo, // Function address (no leading hexadecimal number)
         (WCHAR*)STD_UNDO,
      L"36 Redo",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackRedo, // Function address (no leading hexadecimal number)
         (WCHAR*)STD_REDOW,
      L"06 Delete",                  // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_DELETE,
      L"--",                         // SEPARATOR
      L"06 Find",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_FIND,
      L"06 Replace",                 // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_REPLACE,
      L"--",                         // SEPARATOR
      L"06 Properties",              // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_PROPERTIES,
      L"06 Help",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_HELP,
   nullptr };                           // End of toolbar (nullptr)

WCHAR* ppToolbox[] = {
   L"Toolbox",                        // Toolbox name (no leading hexadecimal number)
      L"24 A",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackHome,  // Function address (no leading hexadecimal number)
         L"IMG_24_A",                 // Button image resource name (defined in resource.rc)
      L"24 X",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackHome,
         L"IMG_24_X",
      L"24 B",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackHome,
         L"IMG_24_B",
      L"24 R",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackHome,
         L"IMG_24_R",
      L"24 E",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackHome,
         L"IMG_24_E",
      L"24 E",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackHome,
         L"IMG_24_E",
      L"24 Z",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackClose, // Function address (no leading hexadecimal number)
         L"IMG_24_Z",
      L"24 E",                        // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackHome,
         L"IMG_24_E",
      L"--",                          // SEPARATOR
      L"26 New",                      // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackNew,   // Function address (no leading hexadecimal number)
         (WCHAR*)STD_FILENEW,         // Common control library image (defined in commctrl.h)
      L"26 Open",                     // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackOpen,  // Function address (no leading hexadecimal number)
         (WCHAR*)STD_FILEOPEN,
      L"06 Save",                     // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_FILESAVE,
      L"--",                          // SEPARATOR
      L"06 Print",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_PRINT,
      L"06 Print Preview",            // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_PRINTPRE,
      L"--",                          // SEPARATOR
      L"06 Cut",                      // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_CUT,
      L"06 Copy",                     // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_COPY,
      L"06 Paste",                    // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_PASTE,
      L"--",                          // SEPARATOR
      L"6E Undo",                     // Button tooltip (popup menu for right mouse click)
         ppEdit[0],                   // Popup menu name
         (WCHAR*)ButtonCallbackUndo,  // Function address (no leading hexadecimal number)
         (WCHAR*)STD_UNDO,
      L"36 Redo",                     // Button tooltip (with leading hexadecimal number)
         (WCHAR*)ButtonCallbackRedo,  // Function address (no leading hexadecimal number)
         (WCHAR*)STD_REDOW,
      L"06 Delete",                   // Button tooltip (with leading hexadecimal number)
         (WCHAR*)STD_DELETE,
   nullptr };                            // End of toolbox (nullptr)

void MouseMoveCallback(AXMEMO aMemo)
{
   switch(aMemo.u)
   {
      case WM_MOUSEMOVE:
      {  // Update keyboard and mouse input memo queue
         DrawStatusbarText(1, L"Mouse position: %d, %d", aMemo.x, aMemo.y);
         break;
      }
      case WM_CHAR:
      case WM_KEYDOWN:
      case WM_KEYUP:
      case WM_LBUTTONDOWN:
      {
         SetFocus(GetDisplayWindow());
         // No break
      }
      case WM_LBUTTONUP:
      case WM_MBUTTONDOWN:
      case WM_MBUTTONUP:
      case WM_RBUTTONDOWN:
      case WM_RBUTTONUP:
      case WM_MOUSEWHEEL:
      {
         DrawStatusbarText(0, L"AXMEMO: %X, %X, %d, %d, %d", aMemo.u, aMemo.v, aMemo.w, aMemo.x, aMemo.y);
         break;
      }
   }
}

// Leading hexadecimal number meaning (for tree view callback function)
// UI_DEFAULT  = 0x80 Specify that this is the default tree item for the specified tree
// UI_GROUPED  = 0x40 Collapse all child items (output only, skip all other flag options)
// UI_CALLBACK = 0x20 Mouse left button clicked (input only)
// UI_COLORED  = 0x10 Delete the tree item (output only, skip all other flag options)
// UI_CHECKED  = 0x08 Expand the list of child items associated with the specified parent item
// UI_BITMAP   = 0x04 Sort the child items of the specified parent item in a tree-view control
// UI_SELECT   = 0x02 Rename the specified item (output only)
// UI_SUBLEVEL = 0x01 Expand all child items (output only, skip all other flag options)

void TreeItemCallback(AXMITE& aMite)
{
   if(aMite.t & UI_CALLBACK) aMite.t |= UI_DEFAULT;
   SpeakText(L"Menu item %d!", aMite.i);
   if(aMite.i == 0) aMite.t |= UI_SUBLEVEL;
   if(aMite.i == 1) aMite.t |= UI_GROUPED;
   if(aMite.i == 3) aMite.t |= UI_CHECKED;
   if(aMite.i == 4) aMite.t &= ~UI_CHECKED;
   if(aMite.i == 5) aMite.t |= UI_SELECT;
   if(aMite.i == 6) aMite.t |= UI_BITMAP;
}

WCHAR* ppTreePopup[] = {
   L"&TreePopup",                     // Menu name (no leading hexadecimal number)
      L"20 Expand All",
         (WCHAR*)TreeItemCallback,
      L"20 Collapse All",
         (WCHAR*)TreeItemCallback,
      L"--",                          // SEPARATOR
      L"60 Expand\tCtrl+E",
         (WCHAR*)TreeItemCallback,
      L"60 Collapse\tCtrl+C",
         (WCHAR*)TreeItemCallback,
      L"E0 Rename\tCtrl+R",
         (WCHAR*)TreeItemCallback,
      L"E0 Sort\tCtrl+S",
         (WCHAR*)TreeItemCallback,
      L"--",                          // SEPARATOR
      L"01 Select",                   // Submenu name (with leading hexadecimal number)
         L"20 Select All\tCtrl+A",    // Submenu item
            (WCHAR*)TreeItemCallback,
         L"20 Select Line\tCtrl+E",   // Submenu item
            (WCHAR*)TreeItemCallback,
         L"20 Select Word\tCtrl+D",   // Submenu item
            (WCHAR*)TreeItemCallback,
         L"20 Select Block\tCtrl+B",  // Submenu item
            (WCHAR*)TreeItemCallback,
      nullptr,                           // End of submenu (nullptr)
      L"40 Column Mode\tAlt+C",
   nullptr };                            // End of menu (nullptr)

void TreeItemSingleClickCallback(AXMITE&)
{
   SpeakText(L"Tree Item %s!", GetTreeviewItem());
}

void TreeItemColorCallback(AXMITE& aMite);

// Leading hexadecimal number meaning (for tree items only)
// UI_DEFAULT  = 0x80 Specify that this is the default tree item for the specified tree
// UI_GROUPED  = 0x40 Specify that a popup menu is provided for the tree item
// UI_CALLBACK = 0x20 Specify that a callback function is provided
// UI_COLORED  = 0x10 Specify that the specified tree item has a custom text color
// UI_CHECKED  = 0x08 Expand the child items associated with the specified parent item
// UI_BITMAP   = 0x04 Specify that a custom bitmap is provided
// UI_SELECT   = 0x02 Ensure that the tree view item is visible (parent item is expanded)
// UI_SUBLEVEL = 0x01 Create a sublevel tree associated with the tree item

WCHAR* ppTree[] = {
   L"Treeview",                               // Tree view name (no leading hexadecimal number)
      L"51 Entry 0",                          // Popup menu, custom text color, and subtree are provided
         ppTreePopup[0],                      // Popup menu name (no leading hexadecimal number)
         (WCHAR*)0x0000FF,                    // Custom text color
         L"44 Upper Case",                    // Tree view item
            ppSearchPopup[0],                 // Popup menu name
            L"IMG_15_SAVEALL1",               // Custom bitmap resource name (defined in resource.rc)
         L"C4 Lower Case",                    // Tree view item
            ppTreePopup[0],
            L"IMG_15_SAVEALL0",
         L"5D Second Level",                  // Subtree name (with second level)
            ppTreePopup[0],
            (WCHAR*)0xFF0000,
            L"IMG_15_SAVEALL0",
            L"24 Not Defined",                // Tree view item
               (WCHAR*)TreeItemColorCallback,
               L"IMG_15_SAVEALL1",
            L"60 Subtree Item",               // Tree view item (callback function is provided)
               ppTreePopup[0],
               (WCHAR*)TreeItemSingleClickCallback,
            L"42 Subtree Test",               // Subtree test
               ppFile[0],
         nullptr,                                // End of subtree (second level)
         L"40 Capitalize",
            ppTreePopup[0],
         L"40 Invert Case",
            ppTreePopup[0],
      nullptr,                                   // End of subtree (nullptr)
      L"53 Open",                             // Tree view item (with subtree)
         ppTreePopup[0],
         (WCHAR*)0x00FF00,
         L"62 Upper Case 2",
            ppTreePopup[0],
            (WCHAR*)TreeItemSingleClickCallback,
         L"40 Lower Case 2",
            ppTreePopup[0],
         L"54 Capitalize 2",
            ppTreePopup[0],
            (WCHAR*)0xFF00FF,
            L"IMG_15_SAVEALL1",
         L"40 Invert Case 2",
            ppTreePopup[0],
      nullptr,                                   // End of subtree (nullptr)
   nullptr };                                    // End of tree view (nullptr)

void TreeItemColorCallback(AXMITE& aMite)
{
   static UINT uClick = 0;
   SetTreeviewColor(uClick++ % 2 ? -1 : 0xFF00FF);
   //SetTreeItemName(L"Clicked %u", uClick);
   if(uClick == 2 || uClick == 4) InsertTreeviewItem(ppTree);
   if(uClick == 6) aMite.t |= UI_COLORED;
   ClearDisplay(0xFF0000FF);
   DrawText(15, 15, L"%p", GetTreeviewItem());
   DrawText(15, 55, L"%s", GetTreeviewItem());
   DrawText(15, 95, L"%X", GetTreeviewColor());
   UpdateDisplay();
   Sleep(500);
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   WORD wWidth = 744, wHeight = 510;

   SetDisplayInputHook(MouseMoveCallback);
   SetDisplayMode(wWidth, wHeight, 1);
   HWND hParent = GetDisplayParent();

   // Call these functions before SetDisplayMode
   AppendMainMenu(ppFile, ppEdit, ppSearch, nullptr);
   HWND hToolbar = SetWindowToolbar(ppToolbar);
   HWND hToolbox = SetWindowToolbox(ppToolbox);
   AppendPopupMenu(ppSearchPopup, ppFile, ppEdit, ppSearch, ppTreePopup, nullptr);
   HWND hTreeView = SetWindowTreeview(150, ppTree);
   SelectPopupMenu(ppFile[0]);
   ReportShortcut(L"..\\Bin\\Shortcut.txt");
   EnableToolbarButton(16, 18, 0);
   EnableToolboxButton(11, 16, 0);

   int iPart[] = { 200, 400, 500, 600, -1 };
   HWND hStatusbar = SetWindowStatusbar(iPart);

   // Enter the game loop
   while(Looping())
   {
      ClearDisplay(0xFF0000FF);
      SetForeColor(0xFFFF0000);
      short sY = 15;
      DrawText(15, sY, L"Parent window size");
      DrawText(300, sY, L"%d", GetWindowSize(hParent).w);
      DrawText(400, sY, L"%d", GetWindowSize(hParent).h);
      SetForeColor(0xFFFF00FF);
      sY += 40;
      DrawText(15, sY, L"Parent client size");
      DrawText(300, sY, L"%d", GetClientSize(hParent).w);
      DrawText(400, sY, L"%d", GetClientSize(hParent).h);
      sY += 40;
      DrawText(15, sY, L"Toolbar size");
      DrawText(300, sY, L"%d", GetWindowSize(hToolbar).w);
      DrawText(400, sY, L"%d", GetWindowSize(hToolbar).h);
      sY += 40;
      DrawText(15, sY, L"Treeview size");
      DrawText(300, sY, L"%d", GetWindowSize(hTreeView).w);
      DrawText(400, sY, L"%d", GetWindowSize(hTreeView).h);
      sY += 40;
      DrawText(15, sY, L"Toolbox size");
      DrawText(300, sY, L"%d", GetWindowSize(hToolbox).w);
      DrawText(400, sY, L"%d", GetWindowSize(hToolbox).h);
      sY += 40;
      DrawText(15, sY, L"Display window size");
      DrawText(300, sY, L"%d", GetDisplayWindowSize().w);
      DrawText(400, sY, L"%d", GetDisplayWindowSize().h);
      SetForeColor(0xFF00FFFF);
      sY += 40;
      DrawText(15, sY, L"Display client size");
      DrawText(300, sY, L"%d", GetDisplayClientSize().w);
      DrawText(400, sY, L"%d", GetDisplayClientSize().h);
      sY += 40;
      DrawText(15, sY, L"Display screen size");
      DrawText(300, sY, L"%d", GetDisplayScreenSize().w);
      DrawText(400, sY, L"%d", GetDisplayScreenSize().h);
      SetForeColor(0xFF00FF00);
      sY += 40;
      DrawText(15, sY, L"Display canvas size");
      DrawText(300, sY, L"%d", GetCanvasWidth());
      DrawText(400, sY, L"%d", GetCanvasHeight());
      SetForeColor(0xFF00FF00);
      sY += 40;
      DrawText(15, sY, L"Statusbar size");
      DrawText(300, sY, L"%d", GetWindowSize(hStatusbar).w);
      DrawText(400, sY, L"%d", GetWindowSize(hStatusbar).h);
      UpdateDisplay(60);
   }
}
