#include "axbreeze.h"
#define MAX_BUFFER 100*1024 // 100K buffer size
void UpdateCameraView(); // Manipulate camera view

XMFLOAT3 fAxis0(0, -0.125, 1);
XMFLOAT3 fAxis1(0, 0.125, -1);
XMFLOAT3 fP, fH, fU;
XMMATRIX mWorld0;
XMMATRIX mWorld1;
HANDLE hImage0;
HANDLE hLight0;
HANDLE hLight1;
HANDLE hObject0;
HANDLE hObject1;
HANDLE hCanvas0;
HANDLE hCanvas1;
HANDLE hSkybox0;
HANDLE hTerrain0;

void UpdateScene()
{
   float t = GetGameTime();

   XMMATRIX mTranslate;
   XMMATRIX mOrbit;
   XMMATRIX mSpin;
   XMMATRIX mScale;
   XMMATRIX mView;
   XMMATRIX mProj;

   mWorld0 = XMMatrixRotationZ(-0.25f*t);
   mScale = XMMatrixScaling(0.5f, 0.5f, 0.5f);
   mTranslate = XMMatrixTranslation(-1, 0.5f, 8);
   mWorld0 = XMMatrixMultiply(XMMatrixMultiply(mWorld0, mScale), mTranslate);
   if(!ShadowMapEnabled())
   {
      DrawTerrain(hTerrain0);
      TurnLight(hLight0, XMFLOAT3(0, 0, 0), fAxis0, 0.002f);
      TurnLight(hLight1, XMFLOAT3(5, 1, 1), fAxis1, 0.005f);
      DrawLightGeometry(hLight0, 0.5, 2, GetLightFarPlane(hLight0), GetLightNearPlane(hLight0));
      DrawLightGeometry(hLight1, 0.5, 3, GetLightFarPlane(hLight1), GetLightNearPlane(hLight1));
      DrawTerrainVolume(hTerrain0, 0xFFFFFFFF, 1);
   }

   mSpin = XMMatrixRotationX(-XM_PIDIV2);
   mOrbit = XMMatrixRotationZ(-0.5f*t);
   mTranslate = XMMatrixTranslation(6, 0.5f, 8);
   mScale = XMMatrixScaling(0.4f, 0.4f, 0.4f);
   mWorld1 = XMMatrixMultiply(XMMatrixMultiply(XMMatrixMultiply(mScale, mSpin), mTranslate), mOrbit);

   SetObjectTransform(hObject0, mWorld0);
   DrawObject(hObject0);
   SetObjectTransform(hObject1, mWorld1);
   DrawObject(hObject1);
   DrawSprite();
   UpdateCameraView();
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   WORD wWidth = 1024, wHeight = 768;

   // Initialize graphics, input, and audio
   SetDisplayMode(wWidth, wHeight);
   hLight0 = GetLightCurrent();
   // Load "helloworld.jpg" to image number 0
   hImage0 = LoadImage(L"..\\Image\\helloworld.jpg");
   BYTE cBuffer[6][MAX_BUFFER]; // Make sure file sizes are smaller than MAX_BUFFER!
   WCHAR* pSource[6] = { L"..\\Image\\skybox10_u.jpg", L"..\\Image\\skybox11_l.jpg",
                         L"..\\Image\\skybox12_f.jpg", L"..\\Image\\skybox13_r.jpg",
                         L"..\\Image\\skybox14_b.jpg", L"..\\Image\\skybox15_d.jpg" };

   BYTE *pRfm[6];
   int iRfm[6];
   for(UINT i = 0; i < 6; i++)
   {
      pRfm[i] = cBuffer[i];
      iRfm[i] = LoadFile(cBuffer[i], MAX_BUFFER, pSource[i]);
   }
   hSkybox0 = LoadSkybox(XMFLOAT3(500, 500, 500), pRfm, iRfm);
   HANDLE hSprite0 = LoadSprite(L"..\\Image\\audrey_kitty.gif");
   SetSprite(hSprite0, 4, 2, 0.1f);
   SetSpriteScale(hSprite0, 25);
   SetSpritePosition(hSprite0, XMFLOAT3(0, -1, 6));
   hTerrain0 = LoadTerrain(XMFLOAT3(40, 40, 6), L"..\\Media\\Heightmap.jpg");
   WCHAR* pTerrain[5] = { L"..\\Media\\snow.png", L"..\\Media\\dirt.png", L"..\\Media\\stone.png",
                          L"..\\Media\\grass.png", L"..\\Media\\blend.jpg" };
   SetTerrainTexture(hTerrain0, pTerrain);
   SetTerrainTiling(hTerrain0, XMFLOAT2(3, 3));
   SetTerrainPosition(hTerrain0, XMFLOAT3(-20, -20, 2));

   hObject0 = LoadObject(L"..\\Model\\LilyPT0.3ds", 0.15f);
   hObject1 = LoadObject(L"..\\Model\\heart.3ds", 0.2f);
   ReportObject(L"..\\Bin\\Materials.txt");

   SetLightType(hLight0, 3, 3);
   SetLightSize(hLight0, XMFLOAT2(20, 15), 3);
   SetLightColor(hLight0, XMFLOAT3(2.5f, 1.5f, 2.5f), 0.125);
   SetLightPosition(hLight0, XMFLOAT3(10, 0, 16));
   hLight1 = CloneLight(hLight0);
   SetLightType(hLight1, 3, 4);
   SetLightColor(hLight1, XMFLOAT3(2.5f, 2.5f, 0.5f), 0.125);
   SetLightCurrent(hLight1);
   SetSpriteBillboard(0, 1, 1, 1);
   SetFogColor(XMFLOAT4(0.5, 0.5, 0.5, 0.5));

   SetCameraPosition(nullptr, XMFLOAT3(0, -20, 15));
   //DisablePhongShading(true);
   //DisableSolidFilling(true);
   //DisableBackCulling(true);
   //DisableAutoSpotlight(true);
   SetShadowMap(1024, 1024);

   // Enter the game loop
   while(Looping())
   {
      PauseGameTime();
      ClearDisplay();
      EnableShadowMap(true);
      UpdateScene();
      EnableShadowMap(false);
      UpdateScene();
      ResumeGameTime();
      float fHeight = GetTerrainHeight(hTerrain0, 40.7f, 30.7f);
      DrawText(500, 20, L"GetTerrainHeight: %f", fHeight);
      DrawShadowMap(XMFLOAT3(-3, -8, 5), XMFLOAT3(-0.25, 0, 0), 2);
      DrawShadowMap(20, 420, 0.25);
      UpdateDisplay(60);
   }
}
