#include "axbreeze.h"
#define MAX_BUFFER 100*1024 // 100K buffer size
void UpdateCameraView(); // Manipulate camera view

XMFLOAT3 fP, fH, fU;
XMFLOAT2 fD;
XMMATRIX mWorld0, mWorld1;
HANDLE hObject1 = nullptr;
HANDLE hObject2 = nullptr;
HANDLE hSkybox1 = nullptr;
HANDLE hCamera1 = nullptr;

void UpdateScene()
{
   ClearDisplay();

   float t = GetGameTime();

   XMMATRIX mTranslate;
   XMMATRIX mOrbit;
   XMMATRIX mSpin;
   XMMATRIX mScale;

   mWorld0 = XMMatrixRotationRollPitchYaw(0.25f*t, 0.0f, 0.0f);
   mScale = XMMatrixScaling(0.75f, 0.75f, 0.75f);
   mTranslate = XMMatrixTranslation(0.0f, 0.0f, -4.5f);
   mWorld0 = XMMatrixMultiply(XMMatrixMultiply(mWorld0, mScale), mTranslate);

   mSpin = XMMatrixRotationX(0.5f*XM_PI);
   mOrbit = XMMatrixRotationZ(0.5f*t);
   mTranslate = XMMatrixTranslation(20.0f, 0.0f, 0.5f);
   mScale = XMMatrixScaling(0.5f, 0.5f, 0.5f);
   mWorld1 = XMMatrixMultiply(XMMatrixMultiply(XMMatrixMultiply(mScale, mSpin), mTranslate), mOrbit);

   DrawSkybox(hSkybox1);
   DrawText(100, 40, L"Camera rotation and movement, use arrow, shift, and control keys");
   SetObjectTransform(hObject1, mWorld0);
   DrawObject(hObject1);
   bool bCamera = CameraIntersects(hCamera1, hObject1);
   DrawObjectVolume(hObject1, bCamera ? 0xFFFF00FF : 0xFFFFFFFF, 2);
   SetObjectTransform(hObject2, mWorld1);
   DrawObject(hObject2);
   DrawObjectVolume(hObject2, CameraIntersects(hCamera1, hObject2) ? 0xFFFF00FF : 0xFFFFFFFF, 2);
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   SetDisplayMode(1024, 768);
   SetCameraPosition(nullptr, XMFLOAT3(0.0f, -40.0f, 5.0f));
   HANDLE hLight0 = GetLightCurrent();

   // Initialize the world matrices
   mWorld0 = mWorld1 = XMMatrixIdentity();

   BYTE cBuffer[6][MAX_BUFFER]; // Make sure file sizes are smaller than MAX_BUFFER!
   WCHAR* pSource[6] = { L"..\\Image\\skybox10_u.jpg", L"..\\Image\\skybox11_l.jpg",
                         L"..\\Image\\skybox12_f.jpg", L"..\\Image\\skybox13_r.jpg",
                         L"..\\Image\\skybox14_b.jpg", L"..\\Image\\skybox15_d.jpg" };

   BYTE *pRfm[6];
   int iRfm[6];
   for(UINT i = 0; i < 6; i++)
   {
      pRfm[i] = cBuffer[i];
      iRfm[i] = LoadFile(cBuffer[i], MAX_BUFFER, pSource[i]);
   }
   pRfm[0] = pRfm[1] = pRfm[4] = nullptr;
   hSkybox1 = LoadSkybox(XMFLOAT3(500.0f, 500.0f, 500.0f), pRfm, iRfm);

   DisableBackCulling(true);
//   hObject1 = LoadObject(L"..\\Model\\LilyPT0.3ds", 0.5f);
   hObject1 = LoadObject(L"..\\Model\\ikea_lack.obj", 0.25f);
//   hObject2 = LoadObject(L"..\\Model\\heart.3ds", 0.5f);
   hObject2 = LoadObject(L"..\\Model\\Chair_Ivar_Ikea.obj", 20.0f);

   hCamera1 = CreateCamera(0.5, 2, 25);
   SetCameraPosition(hCamera1, XMFLOAT3(0.0f, -5.0f, 0.0f));

   SetLightType(hLight0, 3);
   SetLightPosition(hLight0, XMFLOAT3(15.0f, 10.0f, 15.0f));
   SetLightDirection(hLight0, XMFLOAT3(-15.0f, -10.0f, -5.0f));
   SetLightColor(hLight0, XMFLOAT3(2.0f, 2.0f, 2.0f), 0.25);
   //EnablePhongModel(1);
   //EnableFogEffect(1);
   GetLightPosition(hLight0, fP);
   GetLightDirection(hLight0, fU);

   // Enter the main loop
   while(Looping())
   {
      UpdateScene();
      UpdateCameraView();
      TurnCamera(hCamera1, XMFLOAT3(0.0f, 0.0f, 1.0f), 0.001f);
      DrawCameraFrustum(hCamera1, 0xFF00FFFF, 2);
      TurnLight(hLight0, fP, fU, 0.005f);
      DrawLightGeometry(hLight0, 0.75, 3, 4, true);
      DrawTriad(XMFLOAT3(0.0f, 0.0f, 0.0f), XMFLOAT3(1.0f, 0.0f, 0.0f), XMFLOAT3(0.0f, 1.0f, 0.0f), 14.0f, 10);
      UpdateDisplay(0);
   }
}
