#include "axbreeze.h"
#define MAX_BUFFER 900*1024 // 900K buffer size
void UpdateCameraView(); // Manipulate camera view
HANDLE hImage0;
HANDLE hObject1;
HANDLE hSkybox1;
HANDLE hTerrain0;
HANDLE hParticle0;
HANDLE hParticle1;
XMMATRIX mWorld1;
BYTE cBuffer[6][MAX_BUFFER]; // Make sure file sizes are smaller than MAX_BUFFER!

float dt = 0.05f;

void UpdateScene()
{
   ClearDisplay();

   float t = GetGameTime();
   XMMATRIX mTranslate;
   XMMATRIX mOrbit;
   XMMATRIX mSpin;
   XMMATRIX mScale;

   // 2nd Transformation: Orbit around the origin
   mSpin = XMMatrixRotationRollPitchYaw(0.05f*XM_PI, 0.0f, t);
   mOrbit = XMMatrixRotationZ(0.8f*t);
   mTranslate = XMMatrixTranslation(-4.0f, 0.0f, 2.0f);
   mScale = XMMatrixScaling(0.3f, 0.3f, 0.3f);
   mWorld1 = XMMatrixMultiply(XMMatrixMultiply(XMMatrixMultiply(mScale, mSpin), mTranslate), mOrbit);
   SetObjectTransform(hObject1, mWorld1);

   SetParticlePosition(hParticle0, XMFLOAT3(3.0f*cos(0.25f*t), 1.0f+3.0f*sin(0.25f*t), 1.5f));

   DrawObject(hObject1);
   DrawSkybox(hSkybox1);
   DrawTerrain(hTerrain0);

   SetMirrorReflection(0.5);
   DrawMirrorImage(hImage0, XMFLOAT3(-2.0f, 8.0f, 4.0f), XMFLOAT3(0*1.54f, 0.0f, 0.0f), 10);
   DrawObject(hObject1);
   DrawSkybox(hSkybox1);
   DrawTerrain(hTerrain0);
   DrawParticle(hParticle0);
   DrawParticle(hParticle1);
   SetMirrorReflection(0);

   DrawParticle(hParticle0);
   DrawParticle(hParticle1);
}

int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR, int)
{
   // Define an array to hold data
   //BYTE pBuffer[MAX_BUFFER];

   WORD wWidth = 1024, wHeight = 768;

   // Initialize graphics, input, and audio
   SetDisplayMode(wWidth, wHeight);
   SetCameraPosition(nullptr, XMFLOAT3(0.0f, -16.0f, 5.0f), XMFLOAT3(0.0f, 0.0f, 1.0f));

   // Load "helloworld.jpg" to image number 0
   hImage0 = LoadImage(L"..\\Image\\helloworld.jpg");
   hObject1 = LoadObject(L"..\\Model\\RoseRT0.3ds", 0.15f);
   WCHAR* pSkybox[6] = { L"..\\Image\\skybox10_u.jpg", L"..\\Image\\skybox11_l.jpg",
                         L"..\\Image\\skybox12_f.jpg", L"..\\Image\\skybox13_r.jpg",
                         L"..\\Image\\skybox14_b.jpg", L"..\\Image\\skybox15_d.jpg" };

   BYTE *pRfm[6];
   int iRfm[6];
   for(UINT i = 0; i < 6; i++)
   {
      pRfm[i] = cBuffer[i];
     iRfm[i] = LoadFile(cBuffer[i], MAX_BUFFER, pSkybox[i]);
   }
   //pRfm[2] = nullptr;
   hSkybox1 = LoadSkybox(XMFLOAT3(500.0f, 500.0f, 500.0f), pRfm, iRfm);
   hTerrain0 = LoadTerrain(XMFLOAT3(34.0f, 34.0f, 4.0f), L"..\\Media\\Heightmap.jpg", 0, XMFLOAT3(-17.0f, -17.0f, 0.0f));
//   WCHAR* pSource[5] = { L"..\\Media\\stone.png", L"..\\Media\\snow.png",
//      L"..\\Media\\grass.png", L"..\\Media\\dirt.png", L"..\\Media\\blend.jpg" };
   WCHAR* pSource[5] = { L"..\\Media\\stone.png", nullptr,
      nullptr, L"..\\Media\\dirt.png", L"..\\Media\\blend.jpg" };
   SetTerrainTexture(hTerrain0, pSource);
   SetTerrainTiling(hTerrain0, XMFLOAT2(3.0f, 3.0f));

   WCHAR* pShader[4] = { L"..\\Shader\\Fire_jG.cso", L"..\\Shader\\Fire_kG.cso", L"..\\Shader\\Fire_kP.cso",
      L"..\\Media\\flare0.dds" };
   hParticle0 = LoadParticle(5000, pShader);
   SetParticle(hParticle0, XMFLOAT3(0, 0, 0), XMFLOAT3(0.5, 0.5, 1), XMFLOAT3(0, 0, 3));

   pShader[0] = L"..\\Shader\\Rain_jG.cso";
   pShader[1] = L"..\\Shader\\Rain_kG.cso";
   pShader[2] = L"..\\Shader\\Rain_kP.cso";
   pShader[3] = L"..\\Media\\raindrop.dds";
   for(UINT i = 0; i < 4; i++)
   {
      pRfm[i] = cBuffer[i];
      iRfm[i] = LoadFile(cBuffer[i], MAX_BUFFER, pShader[i]);
   }
   hParticle1 = LoadParticle(12000, pRfm, iRfm);
   SetParticle(hParticle1, XMFLOAT3(0, 3, 50), XMFLOAT3(0, 0, 1), XMFLOAT3(-1, 0, -14));

   UpdateDisplay();
   while(GetGameTime() < 1.0f);
   DisableSolidFilling(0);
   DisableBackCulling(true);
   SetFogColor(XMFLOAT4(0.5, 0.5, 0.5, 0.75));

   // Enter the game loop
   while(Looping())
   {
      UpdateScene();
      UpdateCameraView();
      UpdateDisplay(30);
   }
}
